package fxKirjasto;

import static fxKirjasto.MovieDialogController.getFieldID;
import fi.jyu.mit.fxgui.*;
import javafx.application.HostServices;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import kirjasto.Kirjasto;
import kirjasto.Elokuva;
import kirjasto.Arvostelu;
import kirjasto.Genre;
import kirjasto.SailoException;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

/**
 * Kirjaston kayttoliityman ohjain
 * @author Erno Pajala
 * @version 29/01/2025
 */
public class KirjastoGUIController implements Initializable {

    @FXML private ListChooser<Elokuva> chooserElokuvat;
    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;

    // Tekstikentat otetteu yksitellen
    @FXML private TextArea editGenret;
    @FXML private TextArea editComment;
    @FXML private TextField editReview;
    @FXML private GridPane gridElokuva;


    // Alustetaan rajapinta
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        alusta();
    }

    @FXML private void handleHakuehto() {
        if ( elokuvaKohdalla != null ) {
            hae(elokuvaKohdalla.getElokuvaID());
        }
    }

    @FXML private void handleTallenna() {
        tallenna();
    }


    @FXML private void handleAvaa() {
        avaa();
    }


    @FXML private void handlePoistu() {
        tallenna();
        Platform.exit();
    }


    @FXML private void handleLisaaElokuva() {
        lisaaElokuva();
    }


    @FXML private void handleEditElokuva() {
        editElokuva(kentta);
    }


    @FXML private void handlePoistaElokuva() {
        poistaElokuva();
    }


    @FXML private void handleLisaaArvostelu() {
        lisaaArvostelu();
    }


    @FXML private void handleEditArvostelu() {
        editArvostelu();
    }


    @FXML private void handlePoistaArvostelu() {
        poistaArvostelu();
    }


    @FXML private void handleApua() {
        apua();
    }


    @FXML private void handleTietoja() {
        ModalController.showModal(KirjastoGUIController.class.getResource("TietojaView.fxml"), "Tietoja", null, "");
    }


    @FXML
    private void handleThemeChange() {
        altTheme = !altTheme;
        switchTheme();
    }

    /*===================================================FXML=LOPPU===================================================*/

    private String kirjastonnimi = "omakirjasto";
    private Kirjasto kirjasto; // Kaytetaan tata kirjastoa
    private Elokuva elokuvaKohdalla;
    private static final Elokuva apuelokuva = new Elokuva();
    private HostServices hostServices;  // Lisätään HostServices url avausta varten
    private TextField[] edits;
    private int kentta = 0; // Kentta jota editoidaan
    private boolean altTheme = false;


    /**
     * Vaihdetaan teemaa
     */
    private void switchTheme() {
        Scene scene = editReview.getScene(); // otetaan muokattava scene
        if (scene != null) {
            // Poistetaan tyyli
            scene.getStylesheets().clear();

            try {
                // Vaihdetaan teemaa
                URL cssResource;
                if (altTheme) {
                    cssResource = getClass().getResource("altTheme.css");
                } else {
                    cssResource = getClass().getResource("kirjasto.css");
                }

                if (cssResource == null) {
                    throw new RuntimeException("Ei loydetty teemaa");
                }

                scene.getStylesheets().add(cssResource.toExternalForm());
            } catch (Exception e) {
                System.err.println("Virhe teeman lataamisessa " + e.getMessage());
            }
        }
    }


    /**
     * Asetetaan HostServices
     * @param hostServices HostServices
     */
    public void setHostServices(HostServices hostServices) {
        this.hostServices = hostServices;
    }


    // siivotaan pois alustetut jasenet
    private void alusta() {
        // Tyhjennetaan kaikki teksialueet
        chooserElokuvat.clear();
        editReview.clear();
        editComment.clear();
        editGenret.clear();

        // Lisataan jarjesstettavat tekstit comboBoxiin
        cbKentat.clear();
        cbKentat.add("Nimi (A-Ö)", null);
        cbKentat.add("Ohjaaja (A-Ö)", null);
        cbKentat.add("Vuosi (Ylöspäin)", null);
        cbKentat.add("Kesto (Ylöspäin)", null);
        cbKentat.getSelectionModel().select(0);

        // Kun elokuva valitaan chooserista, mennaan naytaElokuvaan
        chooserElokuvat.addSelectionListener(e -> naytaElokuva());

        edits = MovieDialogController.luoKentat(gridElokuva);
        // Laitetaan kentat helpommin muokattavaksi
        for (TextField edit: edits) {
            if (edit != null) {
                edit.setEditable(false);
                edit.setOnMouseClicked(e -> {
                    if (e.getClickCount() > 1) editElokuva(getFieldID(edit, kentta));
                });
                edit.focusedProperty().addListener((a, o, n) -> kentta = getFieldID(edit, kentta));
            }
        }

        editGenret.setOnMouseClicked(e -> {
            if (e.getClickCount() > 1) editElokuva(1);
        });

        // Lisää arvostelukentat helpommin muokattavaksi
        // TODO votaisiin laittaa kohdalle
        editReview.setOnMouseClicked(e -> {
            if (e.getClickCount() > 1) editArvostelu();
        });

        editComment.setOnMouseClicked(e -> {
            if (e.getClickCount() > 1) editArvostelu();
        });
    }


    /**
     * Näyttää listasta valitun elokuvan tiedot
     */
    private void naytaElokuva() {
        elokuvaKohdalla = chooserElokuvat.getSelectedObject(); // Palauttaa valitun elokuva olion
        if (elokuvaKohdalla == null) return; // jos ei ole elokuva oliota, palutuu null

        // laitetaan elokuvan tiedot kenttiin
        MovieDialogController.naytaElokuva(edits, elokuvaKohdalla);

        // Laitetaan elokuvan genret nakyviin paaikkunassa
        String elokuvanGenret = haeGenret();
        editGenret.setText(elokuvanGenret);

        // Naytetaan elokuvan arvostelu
        Arvostelu arvosteluKohdalla = kirjasto.annaArvostelu(elokuvaKohdalla.getElokuvaID());
        if (arvosteluKohdalla != null) {
            editReview.setText(arvosteluKohdalla.getArvosana());
            editComment.setText(arvosteluKohdalla.getKommentti());
        }
        else {
            editReview.clear();
            editComment.clear();
        }

    }


    /**
     * Tietojen tallennus tiedostoon
     */
    private void tallenna() {
        try {
            kirjasto.tallenna(kirjastonnimi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelma tallennuksessa! " + e.getMessage());
        }

    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }


    /**
     * Luetaan annettu kirjasto
     * @param nimi Annettu kirjaston nimi
     */
    private void lueTiedosto(String nimi) {
        kirjastonnimi = nimi;

        ModalController.getStage(hakuehto).setTitle("Kirjasto - " + nimi); // Laitetaan ohjelman nimi
        try {
            kirjasto.lueTiedostosta(nimi); // Luetaan tiedostosta kirjaston tiedot
            hae(0); // Haetaan ensimmainen elokuva
        } catch (SailoException e){
            Dialogs.showMessageDialog(e.getMessage());
        }

    }


    /**
     * Luodaan oletus genret kirjastolle
     */
    private void luoGenret() {
        String[] genreNames = {
                "Animaatio", "Draama", "Fantasia", "Kauhu",
                "Komedia", "Romanssi", "Sci-Fi", "Seikkailu",
                "Toiminta", "Trilleri"
        };

        for (String name : genreNames) {
            Genre genre = new Genre();
            genre.rekisteroi();
            genre.taytaGenre(name);
            kirjasto.lisaa(genre);
        }
    }


    /**
     * Lisätään uusi elokuva
     */
    private void lisaaElokuva() {
        try {
            Elokuva uusi = new Elokuva(); // Muodostetaan uusi elokuva
            uusi = MovieDialogController.kysyElokuva(null, uusi, uusi.ekaKentta(), kirjasto); // Kysytaan kentat
            if ( uusi == null ) return;
            uusi.rekisteroi();
            kirjasto.lisaa(uusi);
            hae(uusi.getElokuvaID());
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
        }
    }


    /**
     * Muokataan elokuvan tietoja
     * @param k Mita kenttaa editoidaan
     */
    private void editElokuva(int k){
        Elokuva elokuvaKohdalla = chooserElokuvat.getSelectedObject(); // Valitaan kohdalla oleva elokuva
        if (elokuvaKohdalla == null) return;

        // Tehdaan kopio elokuvasta jota muokataan
        try {
            Elokuva elokuvaklooni = elokuvaKohdalla.clone();
            // Kysytaan elokuvan tiedot
            elokuvaklooni = MovieDialogController.kysyElokuva(null, elokuvaklooni, k, kirjasto);
            // Jos painetaan "cancel" ei tehda mitaan
            if (elokuvaklooni == null) return;
            // Korvataan tai lisataan elokuva kirjastoon
            kirjasto.korvaaTaiLisaa(elokuvaklooni);
            hae(elokuvaKohdalla.getElokuvaID());
        } catch (CloneNotSupportedException | SailoException e) {
            Dialogs.showMessageDialog("Ongelmia kloonin luomisessa " + e.getMessage());
        }
    }


    /**
     * Poistetaan valittu elokuva
     */
    private void poistaElokuva() {
        Elokuva elokuva = elokuvaKohdalla;
        if ( elokuva == null ) return;

        // Poisto dialogi
        if ( !Dialogs.showQuestionDialog(
                "Poisto", "Poistetaanko elokuva: " + elokuvaKohdalla.getNimi(),
                "Kyllä","Ei") )
            return;

        // poistetaan elokuva
        kirjasto.poista(elokuva);
        int index = chooserElokuvat.getSelectedIndex();
        hae(0);
        chooserElokuvat.setSelectedIndex(index);

    }


    /**
     * Lisätään uusi arvostelu
     * TODO Voitaasin tehda paremmin
     */
    private void lisaaArvostelu() {
        elokuvaKohdalla = chooserElokuvat.getSelectedObject(); // Otetaan elokuva jonka kohdalla ollaan

        // Katsotaan onko elokuva olemassa
        if (elokuvaKohdalla == null) {
            Dialogs.showMessageDialog("Ei elokuvaa");
            return;
        }

        // Katsotaan onko arvostelu jo olemassa
        Arvostelu olemassaOlevaArvostelu = kirjasto.annaArvostelu(elokuvaKohdalla.getElokuvaID());
        if (olemassaOlevaArvostelu != null) {
            Dialogs.showMessageDialog("Arvostelu on jo olemassa");
            return;
        }

        try {
            // Luodaan uusi arvostelu
            Arvostelu uusi = new Arvostelu(); // Muodostetaan uusi arvostelu
            uusi = ReviewDialogController.kysyArvostelu(null, uusi);
            if ( uusi == null ) return;

            uusi.rekisteroi(elokuvaKohdalla.getElokuvaID());
            kirjasto.lisaa(uusi);
            hae(elokuvaKohdalla.getElokuvaID()); // Haetaan uudestaan
        } catch (Exception e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
        }


    }


    /**
     * Muokataan arvostelun tietoja
     * TODO Voitaasin tehda paremmin
     */
    private void editArvostelu() {
        Elokuva elokuvaKohdalla = chooserElokuvat.getSelectedObject(); // Valitaan kohdalla oleva elokuva

        // Katsotaan onko elokuva olemassa
        if (elokuvaKohdalla == null) {
            return;
        }

        // Katsotaan onko arvostelu jo olemassa, jos ei ole ei voida muokata
        Arvostelu olemassaOlevaArvostelu = kirjasto.annaArvostelu(elokuvaKohdalla.getElokuvaID());
        if (olemassaOlevaArvostelu == null) {
            Dialogs.showMessageDialog("Arvostelua ei ole olemassa");
            return;
        }

        // Tehdaan kopio arvostelusta jota muokataan
        try {
            // Kloonataan arvostelu
            Arvostelu muokattu = olemassaOlevaArvostelu.clone();
            muokattu = ReviewDialogController.kysyArvostelu(null, muokattu);
            if (muokattu == null) return;

            // Korvatan arvostelu
            kirjasto.korvaaTaiLisaa(muokattu);
            // Hetaan uudestaan
            hae(elokuvaKohdalla.getElokuvaID());
        } catch (CloneNotSupportedException | SailoException e) {
            Dialogs.showMessageDialog("Ongelmia kloonin luomisessa " + e.getMessage());
        }
    }


    /**
     * Poistetaan valittu elokuva
     */
    private void poistaArvostelu() {
        Elokuva elokuvaKohdalla = chooserElokuvat.getSelectedObject();
        if ( elokuvaKohdalla == null ) return;

        Arvostelu arvosteluKohdalla = kirjasto.annaArvostelu(elokuvaKohdalla.getElokuvaID());
        if (arvosteluKohdalla == null) {
            Dialogs.showMessageDialog("Arvostelua ei ole olemassa");
            return;
        }

        // Poisto dialogi
        if ( !Dialogs.showQuestionDialog(
                "Poisto", "Poistetaanko elokuvan " + elokuvaKohdalla.getNimi() + " arvostelu?",
                "Kyllä","Ei") )
            return;

        // poistetaan elokuva
        kirjasto.poista(arvosteluKohdalla);
        int index = chooserElokuvat.getSelectedIndex();
        hae(0);
        chooserElokuvat.setSelectedIndex(index);
    }


    /**
     * Näytetään ohjelman suunnitelma selaimessa
     */
    private void apua() {
        try {
            URI gitsite = new URI("https://gitlab.jyu.fi/erpajala/ohj2");
            hostServices.showDocument(gitsite.toString());
        } catch (URISyntaxException e) {
            Dialogs.showMessageDialog("Virheellinen URL-osoite: " + e.getMessage());
        }
    }


    /**
     * Kasitellaan kirjaston avaaminen
     * @return Palautetaan true jos annettiin nimi ja false jos painettiin Close tai ei nimea
     */
    public boolean avaa() {
        String uusinimi = KirjastoNimiController.kysyNimi (null, kirjastonnimi);
        if (uusinimi == null) return false;

        kirjastonnimi = uusinimi;

        // Luodaan oletus genret uudelle kirjastolle
        luoGenret();

        lueTiedosto(kirjastonnimi); // Kun painetaan OK mennaan lukemaan tiedosto nimella joka saatiin
        return true;
    }


    /**
     * Asetetaan kaytettava kirjasto
     * @param kirjasto kirjasto jota kaytetaan
     */
    public void setKirjasto(Kirjasto kirjasto) {
        this.kirjasto = kirjasto;

        naytaElokuva();
    }


    /**
     * Haetaan elokuvat uudelleen
     * @param eid mika elokuva valitaan aktiiviseksi
     */
    private void hae(int eid) {
        // Selvitetaan minka jasene kohdalla ollaan
        int eNro = eid;
        if (eNro == 0) {
            Elokuva elokuva = chooserElokuvat.getSelectedObject();
            if (elokuva != null) {
                eNro = elokuva.getElokuvaID();
            }
        }

        // Kentta minka mukaan jarjestetaan
        int k = cbKentat.getSelectionModel().getSelectedIndex() + apuelokuva.ekaKentta();
        String ehto = hakuehto.getText(); // Haettava elokuvan nimi

        chooserElokuvat.clear(); // Tyhjennetaan elokuva lista

        // Lisataan kirjaston elokuvat chooserElokuva nayttoon
        int index = 0;
        Collection<Elokuva> elokuvat = kirjasto.etsi(ehto, k);
        int i = 0;
        for (Elokuva elokuva : elokuvat) {
            if (elokuva.getElokuvaID() == eNro) {
                index = i;
            }
            chooserElokuvat.add(elokuva.getNimi(), elokuva);
            i++;
        }
        chooserElokuvat.setSelectedIndex(index); // Muutos viesti, joka nayttaa elokuvaan
    }


    /**
     * Hakee genret elokuvan kohdalla genreIDn avulla
     * @return Kaikki genret formatoituna merkkijonoksi
     */
    private String haeGenret(){
        List<Genre> genreet = kirjasto.annaGenret(elokuvaKohdalla);
        StringBuilder genreSB = new StringBuilder(); // Genret listattuna

        for (Genre genre : genreet) {
            if (!genreSB.isEmpty()) {
                genreSB.append(", "); // Lisataan pilkku genrejen valiin
            }
            genreSB.append(genre.getGenreName());
        }
        return genreSB.toString();
    }

}