package fxKirjasto;

import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import fi.jyu.mit.ohj2.Mjonot;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;
import kirjasto.Elokuva;
import kirjasto.Genre;
import kirjasto.Kirjasto;
import org.controlsfx.control.CheckComboBox;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

/**
 * Luokka kysyy kayttajalta elokuvan tiedot
 * @author Erno Pajala
 * @version 29/01/2025
 */
public class MovieDialogController implements ModalControllerInterface<Elokuva>, Initializable {

    @FXML private CheckComboBox<String> genreCheckBox;
    @FXML private GridPane gridElokuva;
    @FXML private Label labelVirhe;
    @FXML private TextField genreField;


    // alustaa .fxml-tiedostossa määritellyt attribuutit
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        alusta();
    }


    @FXML private void handleGenreAdd() {
        addGenre();
    }


    @FXML private void handleGenreRemove() {
        String s = genreField.getText();
    }


    @Override
    public Elokuva getResult() {
        return elokuvaKohdalla;
    }


    @FXML private void handleOK() {

        // Katsotaan onko nimi tyhja
        if (elokuvaKohdalla != null && elokuvaKohdalla.getNimi().trim().isEmpty()) {
            naytaVirhe("Nimi ei saa olla tyhjä");
            return;
        }

        ModalController.closeStage(labelVirhe);
    }


    @FXML private void handleCancel() {
        elokuvaKohdalla = null;
        ModalController.closeStage(labelVirhe);
    }


    // Alustaa elokuvan atribuutit
    @Override
    public void setDefault(Elokuva elokuva) {
        elokuvaKohdalla = elokuva;
        naytaElokuva(edits, elokuvaKohdalla);

        // Naytetaan elokuvan genret genreCheckBoxin listassa checkattuna
        naytaElokuvanGenret(elokuvaKohdalla);
    }


    // Kutsutaan kun dialogi on tullut näyttöön
    @Override
    public void handleShown() {
        // Kun tuplaklikataan tiettya kenttaa, laitetaan muokkaus paalle
        kentta = Math.max(apuelokuva.ekaKentta(), Math.min(kentta, apuelokuva.getKenttia() -1));
        edits[kentta].requestFocus();
    }


    /*======================================================================================================*/
    private Elokuva elokuvaKohdalla;
    private TextField[] edits; // Elokuvan fieldit jota taytetaan
    private static final Elokuva apuelokuva = new Elokuva(); // Apuelokuva, jotta tiedetaan kenttien maara
    private int kentta;
    private Kirjasto kirjasto;


    /**
     * Alustetaan
     */
    private void alusta() {
        edits = luoKentat(gridElokuva);

        // laitetaan arvot kenttiin ja tehtaan muutoksen katsoja
        for (TextField edit : edits) {
            if (edit != null) {
                edit.setOnKeyReleased(e -> kasitteleMuutosElokuvaan(edit));
            }
        }

        // Lisataan muutoksen katsoja elokuvan genreille
        genreCheckBox.getCheckModel().getCheckedItems().addListener(
                (javafx.collections.ListChangeListener.Change<? extends String> c) -> updateGenreIDs());
    }


    /**
     * Alustetaan muokattavan kentan indeksin
     * @param kentta kentta
     */
    private void setKentta(int kentta) {
        this.kentta = kentta;
    }


    /**
     * Alustetaan kirjasto
     * @param kirjasto kirjasto
     */
    private void setKirjasto(Kirjasto kirjasto) {
        this.kirjasto = kirjasto;

        // Ladataan kirjaston genret
        lataaGenret();
    }


    /**
     * Luodaan GridPaneen elokluvan tiedot
     * @param gridElokuva mihin tiedot luodaan
     * @return luodut tekstikentat
     */
    public static TextField[] luoKentat(GridPane gridElokuva) {
        gridElokuva.getChildren().clear();
        TextField[] edits = new TextField[apuelokuva.getKenttia()];

        // Kaydaan lapi elokuvan kentat ja luodaan tekstikentat ja labelit gridpaneliin
        for (int i = 0, k = apuelokuva.ekaKentta(); k < apuelokuva.getKenttia(); k++, i++) {
            Label label = new Label(apuelokuva.getKysymys(k));
            gridElokuva.add(label, 0, i); // Lisataan gridiin labeli sarakeeseen
            TextField edit = new TextField(); // Laitetaan texfieldi taulukkoon, kato genren kohdalla jtn
            edits[k] = edit;
            edit.setId("e" + k);
            gridElokuva.add(edit, 1, i);
        }
        return edits;
    }


    /**
     * Lisataan uusi genre kirjastoon
     */
    private void addGenre(){
        String s = genreField.getText();
        if (s == null || s.isEmpty()) return;

        Genre genre = new Genre();
        genre.rekisteroi();
        genre.taytaGenre(s);
        kirjasto.lisaa(genre);

        // Ladataan kirjaston genret uudestaan
        lataaGenret();

        genreField.setText("");
    }


    // TODO: Genrejen lisaamisen olisi voinut tehda samanlailla alustettuina miten muiden tieojen
    /**
     * Lisataan kaikki kirjaston genret genreCheckBoxiin
     */
    private void lataaGenret() {
        if (kirjasto == null) return;

        genreCheckBox.getItems().clear();
        List<Genre> kaikkiGenret = kirjasto.annaKaikkiGenret(); // Haetaan kaikki kirjaston genret

        // Lisataan kirjaston genret nimina genreCheckBoxiin
        for (Genre genre : kaikkiGenret) {
            genreCheckBox.getItems().add(genre.getGenreName());
        }

        naytaElokuvanGenret(elokuvaKohdalla);
    }


    /**
     * Ruksitaan ne genret jotka elokuvalla on jo
     * @param elokuvaKohdalla elokuva, jonka genret katsotaan
     */
    private void naytaElokuvanGenret(Elokuva elokuvaKohdalla) {
        if (elokuvaKohdalla == null || kirjasto == null) return;

        // Siivotaan genret
        genreCheckBox.getCheckModel().clearChecks();

        // Elokuvan genreIDt
        List<Integer> elokuvanGenreIDt = elokuvaKohdalla.getGenreIDs();

        // Valitaan elokuvan genret genreCheckBox listasta
        for (int i = 0; i < genreCheckBox.getItems().size(); i++) {
            for (int genreID : elokuvanGenreIDt) {
                if (i == genreID) {
                    genreCheckBox.getCheckModel().check(i);
                }
            }
        }
    }


    /**
     * Paivitetaan elokuvan genreIDt valituilla genreilla
     */
    private void updateGenreIDs() {
        if (elokuvaKohdalla == null || kirjasto == null) return;

        // Siivotaan elokuvan genreIDt
        elokuvaKohdalla.getGenreIDs().clear();

        // kaydaan lapi valitut genret ja lisataan ne elokuvalle
        List<Integer> valitutGenret = genreCheckBox.getCheckModel().getCheckedIndices();

        for (int genret : valitutGenret) {
            elokuvaKohdalla.setGenreID(genret);
        }
    }


    /**
     * Palautetaan komponentin id:stä saatava luku
     * @param obj tutkittava koponentti
     * @param oletus mika arvo jos id ei ole kunnollinen
     * @return komponentin id lukukuna
     */
    public static int getFieldID(Object obj, int oletus) {
        if ( !(obj instanceof Node node)) {
            return oletus;
        }

        return Mjonot.erotaInt(node.getId().substring(1),oletus);
    }


    /**
     * Kasitellaan muutos elokuvaan
     * @param edit muutettu kentta
     */
    private void kasitteleMuutosElokuvaan(TextField edit) {
        if (elokuvaKohdalla == null) return;
        int k = getFieldID(edit,apuelokuva.ekaKentta());
        String s = edit.getText();
        String virhe;

        // katsotaan kentat
        virhe = elokuvaKohdalla.aseta(k ,s);

        naytaVirhe(virhe);
    }


    /**
     * Naytaa virheen jos kirjoitetaan vaaraa tietoa
     * @param virhe joka naytetaan
     */
    private void naytaVirhe(String virhe) {
        if (virhe == null || virhe.isEmpty()) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().add("Virhe");
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }


    /**
     * Naytetaan kohdalla olevan elokuva tiedot
     * @param edits        Kentat, johon tiedot laitetaan
     * @param elo          elokuva jonka kohdalla ollaan
     */
    public static void naytaElokuva(TextField[] edits, Elokuva elo) {
        if (elo == null) return;

        // Asetetaan tekstit ja tiedot
        for (int k = elo.ekaKentta(); k < elo.getKenttia(); k++) {
            edits[k].setText(elo.anna(k));
        }
    }


    /**
     * Kysytaan elokuvan tiedot ja palautetaan muokattu elokuva
     * @param modalityStage Mihin on modaalinen
     * @param movie Elokuvan teidot milla kentat taytetaan
     * @param kentta jota korostetaan
     * @param kirjasto kirjasto jota kaytetaan
     * @return Muokattu elokuva
     */
    public static Elokuva kysyElokuva(Stage modalityStage, Elokuva movie, int kentta, Kirjasto kirjasto) {
        return ModalController.<Elokuva, MovieDialogController>showModal(
                KirjastoGUIController.class.getResource("MovieDialogView.fxml"),
                "Elokuva", modalityStage, movie,
                ctrl -> { ctrl.setKentta(kentta); ctrl.setKirjasto(kirjasto); }
        );
    }

}
