package fxKirjasto;

import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.stage.Stage;
import kirjasto.Arvostelu;

import java.net.URL;
import java.util.ResourceBundle;

/**
 * Luokka kysyy kayttajalta arvostelun tiedot
 *
 * @author Erno Pajala
 * @version 29/01/2025
 */
public class ReviewDialogController implements ModalControllerInterface<Arvostelu>, Initializable {

    @FXML private TextField editReview;
    @FXML private TextArea editComment;
    @FXML private Label labelVirhe;

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        alusta();
    }


    @Override
    public Arvostelu getResult() {
        return arvosteluKohdalla;
    }


    @FXML private void handleOK (){
        String virhe = arvosteluKohdalla.setArvosana(editReview.getText());
        if (virhe != null) {
            naytaVirhe(virhe);
            return;
        }

        arvosteluKohdalla.setKommentti(editComment.getText());
        ModalController.closeStage(labelVirhe);
    }


    @FXML private void handleCancel (){
        arvosteluKohdalla = null;
        ModalController.closeStage(labelVirhe);
    }


    // Alustetaan arvostelun atribuutit
    @Override
    public void setDefault(Arvostelu oletus) {
        arvosteluKohdalla = oletus;
        naytaArvostelu(arvosteluKohdalla);
    }


    /**
     * Mitea tehdaan kun dialogi naytetaan
     */
    @Override
    public void handleShown() {
        //
    }

    /*===============================================================================*/
    private Arvostelu arvosteluKohdalla;


    /**
     * Alustetaan
     */
    private void alusta(){
        // Alustetaan kentat
        if (editComment != null) {
            editReview.setOnKeyReleased(e -> kasitteleMuutosArvosteluun(editReview));
        }

        if (editComment != null) {
            editComment.setOnKeyReleased(e -> kasitteleMuutosArvosteluun(editComment));
        }
    }


    /**
     * kasitellaan muutos arvosanalle
     * @param review arvosana
     */
    private void kasitteleMuutosArvosteluun(TextField review) {
        if (arvosteluKohdalla == null) return;
        String virhe = arvosteluKohdalla.setArvosana(review.getText());
        naytaVirhe(virhe);
    }


    /**
     * Kasitellaan muutos kommentille
     * @param comment kommentti
     */
    private void kasitteleMuutosArvosteluun(TextArea comment) {
        if (arvosteluKohdalla == null) return;
        arvosteluKohdalla.setKommentti(comment.getText());
        naytaVirhe(null);
    }


    /**
     * Naytaa virheen jos kirjoitetaan vaaraa tietoa
     * @param virhe joka naytetaan
     */
    private void naytaVirhe(String virhe) {
        if (virhe == null || virhe.isEmpty()) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().add("Virhe");
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }


    /**
     * Naytetaan arovstelun tiedot kentissa
     *
     * @param arvostelu elokuvan arvostelu
     */
    public void naytaArvostelu(Arvostelu arvostelu) {
        if (arvostelu == null) return;
        editReview.clear();
        editComment.clear();

        editReview.setText(arvostelu.getArvosana());
        editComment.setText(arvostelu.getKommentti());
    }


    /**
     * Näyttää arvostelu-dialogin ja palauttaa täytetyn arvosteluolion.
     * @param modalityStage mille modaalisuus asetetaan, null = sovellukselle
     * @param oletus Arvostelu-olio, jota muokataan
     * @return null jos painetaan Cancel, muuten täytetty arvostelu
     */
    public static Arvostelu kysyArvostelu(Stage modalityStage, Arvostelu oletus) {
        return ModalController.showModal(
                ReviewDialogController.class.getResource("ReviewDialogView.fxml"),
                "Arvostelu", modalityStage, oletus
        );
    }
}
