package kanta;

/**
 * Luokka numeron tarkistamiseksi
 * @author Erno Pajala
 * @version 11.04.2025
 */
public class NumeroTarkistus {
    public static final String NUMEROT = "0123456789";


    /**
     * Tarkistaa, että merkkijono sisältää vain kokonaislukuja ja että ensimmäinen numero ei ole 0
     * @param jono tutkittava merkkijono
     * @return null jos kelpaa
     * @example <pre name="test">
     *     #PACKAGEIMPORT
     *     NumeroTarkistus numerot = new NumeroTarkistus();
     *     numerot.tarkista("", false) === "Syöte ei saa olla tyhjä";
     *     numerot.tarkista("kfdd", false) === "Anna vain numeroita (0-9)";
     *     numerot.tarkista("12k1k21", false) === "Anna vain numeroita (0-9)";
     *     numerot.tarkista("012", false) === "Ensimmäinen numero ei saa olla 0";
     *     numerot.tarkista("12", false) === null;
     *     numerot.tarkista("100", false) === null;
     *     numerot.tarkista("12", true) === "Arvosanan tulee olla välillä 1-10";
     *     numerot.tarkista("k", true) === "Anna vain numeroita (0-9)";
     * </pre>
     */
    public static String tarkista(String jono, Boolean arvosana) {
        if (jono == null || jono.isEmpty()) {
            return "Syöte ei saa olla tyhjä";
        }

        // Tarkista, että kaikki merkit ovat numeroita
        if (!onkoVain(jono, NUMEROT)) {
            return "Anna vain numeroita (0-9)";
        }

        // Tarkista, että ensimmäinen numero ei ole 0
        if (jono.length() > 1 && jono.charAt(0) == '0') {
            return "Ensimmäinen numero ei saa olla 0";
        }

        // Jos kyseessa on arvosana
        if (arvosana){
            try {
                int arvio = Integer.parseInt(jono);
                if (arvio < 1 || arvio > 10) {
                    return "Arvosanan tulee olla välillä 1-10";
                }
            } catch (NumberFormatException e) {
                return "Arvosanan tulee olla numero välillä 1-10";
            }
        }

        return null;
    }


    /**
     * Onko jonossa vain sallittuja merkkejä
     * @param jono tutkittava jono
     * @param sallitut merkit joita sallitaan
     * @return true jos vain sallittuja, false muuten
     */
    public static boolean onkoVain(String jono, String sallitut) {
        for (int i = 0; i < jono.length(); i++) {
            if (sallitut.indexOf(jono.charAt(i)) < 0) {
                return false;
            }
        }
        return true;
    }
}