package kirjasto;
import fi.jyu.mit.ohj2.Mjonot;
import kanta.NumeroTarkistus;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;


/**
 * Yksittainen elokuva, osaa laittaa genren numeroida genren
 * @author Erno Pajala
 * @version 25.02.2025
 */
public class Elokuva implements Cloneable{
    // Muuttujat
    private int elokuvaID;
    private String nimi = "";
    private String ohjaaja = "";
    private String vuosi = "0";
    private String kestomin = "0";
    private final List<Integer> genreIDs = new ArrayList<>(10); // Käytetään arraylistaa, se voi kasvaa ja kutistua dynaamisesti
    private static int seuraavaNro = 1;

    /**
     * Muodostaja
     */
    public Elokuva() {
        // Ei tarvita viel
    }


    /**
     * Elokuvien vertailija
     */
    public static class Vertailija implements Comparator<Elokuva> {
        private final int k;

        public Vertailija(int k) {
            this.k = k;
        }

        @Override
        public int compare(Elokuva e1, Elokuva e2) {
            return e1.getAvain(k).compareToIgnoreCase(e2.getAvain(k));
        }
    }


    /**
     * Antaa k:n kentän sisällön merkkijonona
     * @param k monenenko kentän sisältö palautetaan
     * @return kentän sisältö merkkijonona
     */
    public String getAvain(int k) {
        return switch (k) {
            case 0 -> "" + elokuvaID;
            case 1 -> nimi.toUpperCase();
            case 2 -> ohjaaja.toUpperCase();
            case 3 -> vuosi;
            case 4 -> kestomin;
            default -> "Virhe";
        };
    }


    /**
     * @return Palauttaa elokuvan ID:n
     */
    public int getElokuvaID() {
        return elokuvaID;
    }


    // Getterit ja setterit
    /**
     * @return Elokuvan nimi
     * @example
     * <pre name="test">
     *   Elokuva Terminator = new Elokuva();
     *   Terminator.taytaElokuva();
     *   Terminator.getNimi() =R= "Terminator.*";
     * </pre>
     */
    public String getNimi() {
        return nimi;
    }


    /**
     * @return palautetaan ohjaaja
     */
    public String getOhjaaja() {
        return ohjaaja;
    }


    /**
     * @return palautetaan vuosi
     */
    public String getVuosi() {
        return vuosi;
    }


    /**
     * @return palautetaan kesto minuuteissa
     */
    public String getKestomin() {
        return kestomin;
    }


    /**
     * @return Elokuvan genreIDs
     */
    public List<Integer> getGenreIDs() {
        return genreIDs;
    }


    /**
     * Annetaan kenttien maara
     * @return montako kenttaa
     */
    public int getKenttia() {
        return 5;
    }


    /**
     * Mika on ensimmainen kentta, jonka tiedot tulis nayttaa ohjelmassa
     * @return ekan kentan ideksi
     */
    public int ekaKentta() {
        return 1;
    }


    /**
     * Palauttaa k:tta elokluvan kenttaa vastaavan kysymyksen
     * @param k kuinka monennen kentan kysymys palautetaan (0-alkuinen)
     * @return k:netta kenttaa vastaava kysymys
     */
    public String getKysymys(int k) {
        return switch (k) {
            case 0 -> "Elokuva ID";
            case 1 -> "Nimi";
            case 2 -> "Ohjaaja";
            case 3 -> "Vuosi";
            case 4 -> "Kesto min";
            default -> "Virhe";
        };
    }


    /**
     * Antaa k:n kentan sisallon merkkijonona
     * @param k kuinka monennen kentan sisalto palautetaan
     * @return kentan sisalto merkkijonona
     */
    public String anna(int k) {
        return switch (k) {
            case 0 -> "" + elokuvaID;
            case 1 -> nimi;
            case 2 -> ohjaaja;
            case 3 -> vuosi;
            case 4 -> kestomin;
            default -> "";
        };
    }


    /**
     * Asettaa elokuvaIDn ja samalla varmistaa etta
     * suraava ID on aina suurempi kun tahan mennessa suurin
     * @param id asetettava ID
     */
    public void setElokuvaID(int id) {
        elokuvaID = id;
        if (elokuvaID >= seuraavaNro){
            seuraavaNro = elokuvaID + 1;
        }
    }


    /**
     * Asettaa nimen ja palauttaa virheen
     * @param s uusi nimi
     * @return null jos ok, muuten virhe
     */
    public String setName(String s) {
        this.nimi = s;
        return null;
    }


    /**
     * Aseta uusi genreID
     * @param gid genren ID
     */
    public void setGenreID(int gid) {
        genreIDs.add(gid);
    }


    /**
     * Otetaan tiedoston rivista GenreIDs patka ja otetaan siita genreID:t yksitellen ja
     * lisataan ne genreIDs taulukkoon
     * @param IDs Tiedoston rivissa oleva GenreIDs patka joka erotellaan, ei sialla ] merkkia
     */
    public void setGenreIDs(StringBuilder IDs) {
        Mjonot.erota(IDs, '[');
        while (!IDs.isEmpty()){
            int IDD = Mjonot.erotaInt(IDs, 0);
            genreIDs.add(IDD);
            Mjonot.erota(IDs, ',');
        }
    }


    /**
     * Asetetaan uudet tiedot elokuvan kentiin
     * @param k kentta mihin asetetaan
     * @param s uusi arvo
     * @return jos tuelee virhe
     */
    public String aseta(int k, String s) {
        String virhe;
        switch (k) {
            case 1:
                this.nimi = s; return null;
            case 2:
                this.ohjaaja = s; return null;
            case 3:
                virhe = NumeroTarkistus.tarkista(s, false);
                // Jos saadaan virhe palautetaan se, eika vaiheta vuotta
                if (virhe != null)
                    return virhe;
                this.vuosi = s;
                return null;
            case 4:
                virhe = NumeroTarkistus.tarkista(s, false);
                // Jos saadaan virhe palautetaan se, eika vaiheta kestoa
                if (virhe != null)
                    return virhe;
                this.kestomin = s;
                return null;
            default:
                return "Tuntematon kenttä";
        }
    }


    /**
     * Antaa elokuvalle seuraavan ID:n
     * @return elokuvan uusi ID
     * @example
     * <pre name="test">
     *   Elokuva inception = new Elokuva();
     *   inception.getElokuvaID() === 0;
     *   inception.rekisteroi();
     *   Elokuva terminator = new Elokuva();
     *   terminator.rekisteroi();
     *   int n1 = inception.getElokuvaID();
     *   int n2 = terminator.getElokuvaID();
     *   n1 === n2-1;
     * </pre>
     */
    public int rekisteroi(){
        elokuvaID = seuraavaNro;
        seuraavaNro++;
        return elokuvaID;
    }


    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot elokuvalle
     */
    public void taytaElokuva() {
        int randNum = (int)(Math.random() * 4) + 1;
        nimi = "Terminator "  + randNum;
        ohjaaja = "James Cameron";
        vuosi = "1987";
        kestomin = "120";
        genreIDs.add(0);
        genreIDs.add(1);

    }


    /**
     * Tulsotetaan elokuvan tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println("ID: " + String.format("%03d", elokuvaID, 3));
        out.println("Nimi: " + nimi);
        out.println("Ohjaaja: " + ohjaaja);
        out.println("Vuosi: " + vuosi);
        out.println("Kesto (min): " + kestomin);
        out.println("Genre IDs: " + genreIDs);
        out.println("-----------------------------");
    }


    /**
     * Tulostetaan elokuvan tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }


    /**
     * Selvitaa elokuvan tiedot | erotellusta merkkijonosta ja lisaa ne elokuvalle
     * Pitaa huolen etta seuraava ID on suurempi kuin tuleva ID
     * @param rivi josta elokuvan tiedot otetaan
     * @example
     * <pre name="test">
     *   Elokuva elokuva = new Elokuva();
     *   elokuva.parse(" 4 | Terminator | James Cameron | 1987");
     *   elokuva.getElokuvaID() === 4;
     *   elokuva.toString().startsWith("4 | Terminator | James Cameron | 1987 |") === true;
     *   elokuva.rekisteroi();
     *   int n = elokuva.getElokuvaID();
     *   elokuva.parse(""+(n+20)); // Otetaan merkkijonosta vain elokuvaID
     *   elokuva.rekisteroi(); // ja tarkistetaan että seuraavalla kertaa tulee yhta isompi
     *   elokuva.getElokuvaID() === n+20+1;
     * </pre>

     */
    public void parse(String rivi){
        var sb = new StringBuilder(rivi);
        // Koitetaan erottaa, jos ei pystyta kaytetaan alkuperaista arvoa
        setElokuvaID(Mjonot.erota(sb, '|', getElokuvaID())); // Varmistetaan, etta suraava ID on aina suurempi
        nimi = Mjonot.erota(sb, '|', nimi);
        ohjaaja = Mjonot.erota(sb, '|', ohjaaja);
        vuosi = Mjonot.erota(sb, '|', vuosi);
        kestomin = Mjonot.erota(sb, '|', kestomin);
        setGenreIDs(new StringBuilder(Mjonot.erota(sb, ']', ""))); // Laitetaan GenreIDt elokuvaan
    }


    /**
     * Palauttaa elokuvan teidot merkkijonona jonka voi tallentaa tiedostoon
     * @return elokuvan tiedot
     * @example <pre name="test">
     * Elokuva elokuva = new Elokuva();
     * elokuva.parse("4 | Terminator | James Cameron | 1987");
     * elokuva.toString().startsWith(4 | Terminator | James Cameron | 1987 |") === true;
     * </pre>
     */
    @Override
    public String toString() {
        return getElokuvaID() + " | " +
                getNimi() + " | " +
                getOhjaaja() + " | " +
                getVuosi() + " | " +
                getKestomin() + " | " +
                getGenreIDs();
    }


    /**
     * Tehdaan klooni elokuvasta
     * @return Kloonattu elokuva
     * @throws CloneNotSupportedException
     * @example <pre name="test">
     * #THROWS CloneNotSupportedException
     *   Elokuva elokuva = new Elokuva();
     *   elokuva.parse("3 | Terminator ");
     *   Elokuva kopio = elokuva.clone();
     *   kopio.toString() === elokuva.toString();
     *   elokuva.parse("4 | Terminator 2 ");
     *   kopio.toString().equals(elokuva.toString()) === false;
     * </pre>
     */
    @Override
    public Elokuva clone() throws CloneNotSupportedException {
        Elokuva uusi;
        uusi = (Elokuva)super.clone();
        // Ehkä pitää sijoittaa arvot yksitellen uuteen elokuvaan genreID:ttne takia
        return uusi;
    }

    /**
     * Testataan
     * @param args Ei kaytossa
     */
    public static void main(String[] args) {
        Elokuva Terminator = new Elokuva();
        Elokuva Terminator2 = new Elokuva();

        Terminator.rekisteroi(); // Saa ID arvon 1
        Terminator2.rekisteroi(); // Saa ID arvon 2

        Terminator.tulosta(System.out);
        Terminator.taytaElokuva();
        Terminator.tulosta(System.out);

        Terminator2.tulosta(System.out);
        Terminator2.taytaElokuva();
        Terminator2.tulosta(System.out);
    }
}
