package kirjasto;

import fi.jyu.mit.ohj2.Mjonot;

import java.io.*;
/**
 * Genre, joka huolelhtii ID:sta
 * @author Erno Pajala
 * @version 08.03.2025
**/
public class Genre {
    // Tulevaisuudessa voidaan lisata uusien genrejen lisäys käyttöliittymästä
    private int genreID;
    private String genreName = "";
    private static int seuraavaNro = 0;


    /**
     * Alustetaan arvostelu. Toistaiseksi ei viela mitaan
     */
    public Genre() {
        //
    }

    /**
     * Alustetaan tietty genre
     */
    public Genre(int genreID) {
        this.genreID = genreID;
    }


    // Getteritä
    /**
     * @return Generen nimi
     */
    public String getGenreName() {
        return genreName;
    }


    /**
     * @return Genren ID
     */
    public int getGenreID() {
        return genreID;
    }


    /**
     * Asettaa genreIDn ja samalla varmistaa etta
     * seuraava numero on aina suurempi kuin tahan mennessä suurin
     * @param gid asetettava genreID
     */
    private void setGenreID(int gid) {
        genreID = gid;
        if ( genreID >= seuraavaNro ) seuraavaNro = genreID + 1;
    }


    /**
     * Antaa genrelle seuraavan rekisterinumeron.
     * <pre name="test">
     *   Genre gen = new Genre();
     *   gen.getGenreID() === 0;
     *   gen.rekisteroi();
     *   Genre gen2 = new Genre();
     *   gen2.rekisteroi();
     *   int n1 = gen.getGenreID();
     *   int n2 = gen2.getGenreID();
     *   n1 === n2-1;
     * </pre>
     */
    public int rekisteroi() {
        genreID = seuraavaNro;
        seuraavaNro++;
        return genreID;
    }


    /**
     * Apumetodi jolla voidaan täyttää tesiarvo genrelle
     * @param arvo Valitaan testi genre
     */
    public void taytaGenre(String arvo) {
        genreName = arvo;
    }


    /**
     * Tulostetaan genren tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println("Genre ID: " + genreID);
        out.println("Genre: " + genreName);
    }


    /**
     * Tulostetaan genren tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }


    /**
     * Selvitaa genren tiedot | erotellusta merkkijonosta ja lisaa ne genrelle
     * Pitaa huolen etta seuraava ID on suurempi kuin tuleva ID
     * @param rivi josta genren tiedot otetaan
     * @example
     * <pre name="test">
     *   Genre gen = new Genre();
     *   gen.parse(" 1 | Animaatio");
     *   gen.getGenreID() === 1;
     *   gen.rekisteroi();
     *   int n = gen.getGenreID();
     *   gen.parse(""+(n+20)); // Otetaan merkkijonosta vain genreID
     *   gen.rekisteroi(); // ja tarkistetaan että seuraavalla kertaa tulee yhta isompi
     *   gen.getGenreID() === n+20+1;
     * </pre>
     */
    public void parse(String rivi){
        var sb = new StringBuilder(rivi);
        // Koitetaan erottaa, jos ei pystyta kaytetaan alkuperaista arvoa
        setGenreID(Mjonot.erota(sb, '|', getGenreID())); // Varmistetaan, etta suraava ID on aina suurempi
        genreName = Mjonot.erota(sb, '|', genreName);
    }


    /**
     * Palauttaa genren teidot merkkijonona jonka voi tallentaa tiedostoon
     * @return genren tiedot
     */
    @Override
    public String toString() {
        return getGenreID() + " | " +
                getGenreName();
    }


    /**
     * Testiohjelma Arvosteluille
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Genre gen = new Genre();
        Genre gen2 = new Genre();
        gen.rekisteroi();
        gen2.rekisteroi();
        gen.taytaGenre("Toiminta");
        gen2.taytaGenre("Sci-Fi");

        gen.tulosta(System.out);
        gen2.tulosta(System.out);
    }
}
